@@ -1,8 +1,8 @@
 //! DER decoder.
 
 use crate::{
-    Any, BitString, Choice, Decodable, ErrorKind, GeneralizedTime, Ia5String, Length, Null,
-    OctetString, PrintableString, Result, Sequence, UtcTime, Utf8String,
+    Any, BitString, Choice, ContextSpecific, Decodable, ErrorKind, GeneralizedTime, Ia5String,
+    Length, Null, OctetString, PrintableString, Result, Sequence, UtcTime, Utf8String,
 };
 use core::convert::{TryFrom, TryInto};
 
@@ -90,6 +90,11 @@ impl<'a> Decoder<'a> {
         self.decode()
     }
 
+    /// Attempt to decode an `OPTIONAL` ASN.1 `ANY` value.
+    pub fn any_optional(&mut self) -> Result<Option<Any<'a>>> {
+        self.decode()
+    }
+
     /// Attempt to decode ASN.1 `INTEGER` as `i8`
     pub fn int8(&mut self) -> Result<i8> {
         self.decode()
@@ -125,6 +130,16 @@ impl<'a> Decoder<'a> {
         self.decode()
     }
 
+    /// Attempt to decode an ASN.1 `CONTEXT-SPECIFIC` field.
+    pub fn context_specific(&mut self) -> Result<ContextSpecific<'a>> {
+        self.decode()
+    }
+
+    /// Attempt to decode an `OPTIONAL` ASN.1 `CONTEXT-SPECIFIC` field.
+    pub fn context_specific_optional(&mut self) -> Result<Option<ContextSpecific<'a>>> {
+        self.decode()
+    }
+
     /// Attempt to decode an ASN.1 `GeneralizedTime`.
     pub fn generalized_time(&mut self) -> Result<GeneralizedTime> {
         self.decode()
