@@ -326,7 +326,7 @@
 #![doc(
     html_logo_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
     html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
-    html_root_url = "https://docs.rs/der/0.3.4"
+    html_root_url = "https://docs.rs/der/0.3.5"
 )]
 #![forbid(unsafe_code, clippy::unwrap_used)]
 #![warn(missing_docs, rust_2018_idioms, unused_qualifications)]
@@ -357,6 +357,7 @@ pub use crate::{
         any::Any,
         bit_string::BitString,
         choice::Choice,
+        context_specific::ContextSpecific,
         generalized_time::GeneralizedTime,
         ia5_string::Ia5String,
         null::Null,
@@ -375,7 +376,7 @@ pub use crate::{
     header::Header,
     length::Length,
     message::Message,
-    tag::{Tag, Tagged},
+    tag::{Class, Tag, Tagged},
 };
 
 pub(crate) use crate::byte_slice::ByteSlice;
